#ifndef VEDIRECT_VETI_H
#define VEDIRECT_VETI_H

#include "vedirect.h"
#include "veti.h"
#include "prod_class.h"

enum VdVetiActionType { VdVetiGet, VdVetiSet };

typedef struct {
	enum VdVetiActionType actionType;
	VetiId id;
	VeVariantValue value; /* Used in VdVetiSet */
} VdVetiAction;

typedef enum { VdVetiActionListFull } VdVetiError;
typedef void (*VdVetiErrorNotifyFunc)(VdVetiError error, void * context);

typedef struct {
	void * context;
	VdVetiErrorNotifyFunc errorNotify;
	SubHandles veDirectVetiHandle;
	unsigned int actionsHead;
	unsigned int actionsCount;
	VdVetiAction actions[CFG_VD_VETI_ACTIONS];
	VeDirectInstance * vdInstance;
	VetiInstance * vetiInstance;
} VdVetiInstance;

#ifdef __cplusplus
extern "C" {
#endif

/* Should be called once */
void vdVetiInit(VdVetiInstance * instance, VeDirectInstance * vd, VetiInstance * veti, void * context);
static inline void vdVetiConnectErrorNotify(VdVetiInstance * instance, VdVetiErrorNotifyFunc func) { instance->errorNotify = func; }

VeProdClasses vdVetiClass(un16 productId);
static inline veBool vdVetiActionsFull(VdVetiInstance * instance) { return instance->actionsCount >= CFG_VD_VETI_ACTIONS; }

/* These will be registrered by vedirect_veti,
 * so when registering your own,
 * make sure to call these as the first during your own callback */
void vdVetiTextItemNotify (const VeRegId id, const VeVariantValue * value, const VeDataBasicType type, VdVetiInstance * instance);
void vdVetiHexMessageNotify (const VeRegId id, const un8 * const data, const un8 length, const un8 flags, const veBool async, VdVetiInstance * const instance);
void vdVetiStatusChange(VdDirectStatus status, VdVetiInstance * instance);
void vdVetiHexErrorNotify(const VdHexError error, VdVetiInstance * instance);

#ifdef __cplusplus
}
#endif

#endif // VEDIRECT_VETI_H
