#ifndef _VETEXT_H_
#define _VETEXT_H_

#include <velib/types/types.h>
#include <velib/ve_regs.h>
#include <velib/types/variant.h>

#define SER_LENGTH		20
#define PROD_LENGTH		21
#define BMV_LENGTH		6

typedef struct
{
	veBool valid:1;
	veBool updated:1;
	veBool hex:1;
	VeVariantValue value;
} VeTagCache;

typedef struct
{
	const char * const tagName;
	const VeRegId regId;
	const un16 cacheIndex;
	const sn16 divider;
	const sn16 multiplier;
	const un16 rounder; /* Also used for the buffer length in case of VE_STR */
	const VeDataBasicType type;
} VeTagRecord;

typedef struct {
#if CFG_VETEXT_WITH_BMV
	VeTagCache alarm;
	VeTagCache bmv;
	VeTagCache cah;
	VeTagCache dm;
	VeTagCache h1;
	VeTagCache h2;
	VeTagCache h3;
	VeTagCache h4;
	VeTagCache h5;
	VeTagCache h6;
	VeTagCache h7;
	VeTagCache h8;
	VeTagCache h9;
	VeTagCache h10;
	VeTagCache h11;
	VeTagCache h12;
	VeTagCache h13;
	VeTagCache h14;
	VeTagCache h15;
	VeTagCache h16;
	VeTagCache h17;
	VeTagCache h18;
	VeTagCache p;
	VeTagCache soc;
	VeTagCache t;
	VeTagCache ttg;
	VeTagCache vm;
	VeTagCache vs;
#endif
#if CFG_VETEXT_WITH_MPPT
	VeTagCache err;
	VeTagCache h19;
	VeTagCache h20;
	VeTagCache h21;
	VeTagCache h22;
	VeTagCache h23;
	VeTagCache hsds;
	VeTagCache il;
	VeTagCache load;
	VeTagCache ppv;
	VeTagCache vpv;
#endif
#if CFG_VETEXT_WITH_INVERTER
	VeTagCache acouti;
	VeTagCache acoutv;
	VeTagCache mode;
	VeTagCache warn;
#endif
#if CFG_VETEXT_WITH_BMV || CFG_VETEXT_WITH_MPPT
	VeTagCache prod;
	VeTagCache ver;
	VeTagCache i;
	VeTagCache relay;
#endif
#if CFG_VETEXT_WITH_BMV || CFG_VETEXT_WITH_INVERTER
	VeTagCache ar;
#endif
#if CFG_VETEXT_WITH_MPPT || CFG_VETEXT_WITH_INVERTER
	VeTagCache cs;
#endif
	VeTagCache fw;
	VeTagCache pid;
	VeTagCache ser;
	VeTagCache v;
} VeNamedTagCache;

typedef union {
	VeNamedTagCache named;
	VeTagCache array[sizeof(VeNamedTagCache) / sizeof(VeTagCache)];
} VdTagValues;

typedef struct {
	VdTagValues tagValues;
	char ser[SER_LENGTH];
	char prod[PROD_LENGTH];
#if CFG_VETEXT_WITH_BMV
	char bmv[BMV_LENGTH];
#endif
} VdTextInstance;

void veTextInit(VdTextInstance * instance);
void veTextRxEvent(VdTextInstance * instance, char const * name, char const * value);
void veTextFrameEndEvent(VdTextInstance * instance, veBool valid);

#endif // _VETEXT_H_
