#ifndef _VETI_H_
#define _VETI_H_

#include <velib/velib_config_app.h>
#include <velib/types/variant.h>
#include <velib/types/types.h>

#include "veti_items.h"
#include "prod_class.h"

/*
 * Constants
 */

/*
 * Typedefs
 */
 typedef enum {
	subHandle1 = 0, subHandle2 = 1, subHandle3 = 2, noSubHandle = 0xFF
} SubHandles;

typedef struct {
	veBool reserved:5;
	veBool subscriber3:1;
	veBool subscriber2:1;
	veBool subscriber1:1;
} Subscribers;

typedef void (*SubReply)(void * context, VetiId id, const VeVariantValue * value, veBool valid);
typedef void (*PubReqFunc)(void * context, VetiId id);

#define MAX_NUM_SUBSCRIBERS 3

typedef struct {
	Subscribers subscribers;
	Subscribers subToAllItems;
	SubReply subReplyFunc[MAX_NUM_SUBSCRIBERS];
	void * subscriberContext[MAX_NUM_SUBSCRIBERS];
	PubReqFunc pubReqFunc;
	void * publisherContext;

	VetiData veItems;
} VetiInstance;

/*
 * API functions
 */
#ifdef __cplusplus
extern "C" {
#endif

static inline VeDataBasicType vetiToVeType(VetiTypes type)
{
	switch (type) {
	case VETI_SN8:
		return VE_SN8;
	case VETI_UN8:
		return VE_UN8;
	case VETI_SN16:
		return VE_SN16;
	case VETI_UN16:
		return VE_UN16;
	case VETI_SN32:
		return VE_SN32;
	case VETI_UN32:
		return VE_UN32;
	default:
		return VE_BUF;
	}
}

void vetiInit(VetiInstance * instance);
SubHandles vetiConnect(VetiInstance * instance);
veBool vetiConnectSubscriber(VetiInstance * instance, const SubHandles handle, SubReply replyFunc, void * userContext);
void vetiConnectPublischer(VetiInstance * instance, const PubReqFunc reqFunc, void * userContext);
veBool vetiPublish(VetiInstance * instance, const SubHandles handle, const VetiId id, const VeVariantValue * value);
veBool vetiPublishRequest(VetiInstance * instance, const SubHandles handle, VetiId id);
veBool vetiInvalidateById(VetiInstance * instance, const SubHandles handle, VetiId id);
veBool vetiSubscribe(VetiInstance * instance, const SubHandles handle, const VetiId id);
void vetiSubscribeAll(VetiInstance * instance, const SubHandles handle);
veBool vetiUnSubscribe(VetiInstance * instance, const SubHandles handle, const VetiId id);
void vetiUnSubscribeAll(VetiInstance * instance, const SubHandles handle);
veBool vetiGetValue(VetiInstance * instance, const VetiId id, VeVariantValue * const value);
veBool vetiIsValid(VetiInstance * instance, const VetiId id);
static inline void vetiSetClass(VetiInstance * instance, VeProdClasses prodClass) { vetiItemInit(&instance->veItems, prodClass); }

static inline VeTinyItem * vetiGetItem(VetiInstance * instance, VetiId id) { return vetiItemFind(&instance->veItems.items, id); }
void vetiPublishItem(VetiInstance * instance, const SubHandles handle, const VetiId id, VeTinyItem * item);

static inline VetiTypes	vetiGetType(VetiInstance * instance, VetiId id) { const VeTinyItem * const item = vetiGetItem(instance, id); if (item) return item->properties.type; else return VETI_PTR; }

/* Future use */
//veBool vetiSubscribeList(VetiHandle handle, VetiId idList[]);
//veBool vetiUnSubscribeList(VetiHandle handle, VetiId idList[]);

#ifdef __cplusplus
}
#endif

#endif /* _VETI_H_ */
