#ifndef VEDIRECT_COMMON_H
#define VEDIRECT_COMMON_H

#include <velib/base/base.h>

typedef enum {
	VETI_UNKNOWN,
	VETI_SN8, VETI_UN8,
	VETI_SN16, VETI_UN16,
	VETI_SN32, VETI_UN32,
	VETI_PTR
} VetiTypes;

typedef struct {
	VetiTypes type:3;
	veBool valid:1; /* This has the of whether the item has been published or not */
	veBool request:1;
	veBool subscriber3:1;
	veBool subscriber2:1;
	veBool subscriber1:1;
} VetiProperties;

typedef struct {
	VetiProperties properties;
	VeVariantValue value;
} VeTinyItem;

typedef struct {
	VeTinyItem prodClass;
	VeTinyItem prodId;
	VeTinyItem firmwareVersion;
	VeTinyItem serial;
	VeTinyItem factoryDefault;
	VeTinyItem clearHistory;
	VeTinyItem capabilities1;
#if CFG_VETEXT_WITH_MPPT || CFG_VETEXT_WITH_INVERTER
	VeTinyItem state;
#endif
#if CFG_VETEXT_WITH_INVERTER
	VeTinyItem mode;
	VeTinyItem warningReason;
#endif
#if CFG_VETEXT_WITH_BMV || CFG_VETEXT_WITH_INVERTER
	VeTinyItem alarmReason;
#endif
#if CFG_VETEXT_WITH_BMV || CFG_VETEXT_WITH_MPPT
	VeTinyItem relayState;
#endif
#if CFG_VEDIRECT_LAST_MESSAGE
	VeTinyItem lastMessage;
#endif
} DeviceInfo;

typedef struct {
	VeTinyItem voltage;
#if CFG_VETEXT_WITH_BMV || CFG_VETEXT_WITH_MPPT
	VeTinyItem current;
	VeTinyItem power;
#endif
#if CFG_VETEXT_WITH_BMV
	VeTinyItem temperature;
	VeTinyItem midVoltage;
	VeTinyItem midVoltageDev;
#endif
} Battery;

typedef struct {
	VeTinyItem voltage;
	VeTinyItem current;
} Ac;

#define LAST_MESSAGE_LENGTH		CFG_VEHEX_RX_DATA_LEN

typedef un16 VetiId;

typedef struct {
	VetiId id;
	un8 data[LAST_MESSAGE_LENGTH];
} LastMessage;

#endif // VEDIRECT_COMMON_H
