#ifndef VEDIRECT_ITEMS_H
#define VEDIRECT_ITEMS_H

#include <velib/base/base.h>
#include <velib/types/variant.h>

#include "veti_common.h"
#include "veti_bmv.h"
#include "veti_mppt.h"
#include "prod_class.h"

/* VetiItems should only contain VeTinyItems.
 * Other data should be placed in VetiData.
 */
typedef struct {
	DeviceInfo device;
	Battery battery;
#if CFG_VETEXT_WITH_INVERTER
	Ac ac;
#endif
	union {
#if CFG_VETEXT_WITH_BMV
		VeBmv bmv;
#endif
#if CFG_VETEXT_WITH_MPPT
		VeMppt mppt;
#endif
	} product;
} VetiItems;

typedef struct {
	VetiItems items;

	char ser[11];

	#if CFG_VEDIRECT_LAST_MESSAGE
	LastMessage lastMessage;
	#endif

	#if CFG_VEDIRECT_WITH_HISTORY
	MpptDayHistory daysHistory[DAY_HISTORY_LENGTH];
	#elif CFG_VEDIRECT_WITH_HISTORY_DAY0
	MpptDayHistory daysHistory[1];
	#endif
	#if CFG_VEDIRECT_WITH_HISTORY || CFG_VEDIRECT_WITH_HISTORY_TOTAL
	MpptTotalsHistory totalsHistory;
	#endif
} VetiData;

/*
 * API functions
 */
void vetiItemInit(VetiData * vetiData, VeProdClasses productClass);
VeTinyItem * vetiItemFind(VetiItems * items, VetiId id);
static inline VeTinyItem * vetiItemBegin(VetiItems * items) { return (VeTinyItem *) items; }
static inline VeTinyItem * vetiItemEnd(VetiItems * items) { return vetiItemBegin(items) + (sizeof(VetiItems) / sizeof(VeTinyItem)); }
static inline VeTinyItem * vetiItemNext(VeTinyItem * item) { return item+1; }

#endif // VEDIRECT_ITEMS_H
