#ifndef _VEDIRECT_MPPT_H_
#define _VEDIRECT_MPPT_H_

#include "veti_common.h"

typedef struct {
	VeTinyItem voltage;
	VeTinyItem current;
	VeTinyItem power;
} MpptPanel;

typedef struct {
	VeTinyItem voltage;
	VeTinyItem current;
	VeTinyItem power;
	VeTinyItem userYield;
	VeTinyItem todayYield;
	VeTinyItem todayPmax;
	VeTinyItem yesterdayYield;
	VeTinyItem yesterdayPmax;
} MpptCharger;

typedef struct {
	VeTinyItem voltage;
	VeTinyItem current;
	VeTinyItem power;
} MpptLoad;

typedef struct {
	VeTinyItem error;
	VeTinyItem load;
} MpptState;

#define LAST_CHARGER_ERROR		0
#define PREV_CHARGER_ERROR		1
#define PREVPREV_CHARGER_ERROR	2
#define OLDEST_CHARGER_ERROR	3

typedef union {
	un8 chargerErrors[4];
	un32 allChargerErrors;
} MpptErrorHistroy;

typedef struct {
	un8	extension;
	un8 daysAvailable;
	MpptErrorHistroy errors;
	un32 userYield; // 10Wh, Resettable
	un32 systemYield; // 10Wh
	un16 maxPanelVoltage; // 10mV
	un16 maxBatteryVoltage; // 10mV
	un16 minBatteryVoltage; // 10mV
} MpptTotalsHistory;

typedef struct {
	un8	extension;
	un32 yield; // 10Wh
	un32 consumed; // 10Wh, load output only, reads as 0xFFFF for other models
	un16 maxBatteryVoltage; // 10mV
	un16 minBatteryVoltage; //10mV
	MpptErrorHistroy errors;
	un16 timeInBulk; // 1min
	un16 timeInAbsorption; // 1min
	un16 timeInFloat; // 1min
	un32 maxPower; // 1W
	un16 maxBatteryCurrent; // 100mA
	un16 maxPanelVoltage; // 10mV
	un16 daySequenceNumber; // 0..365
} MpptDayHistory;

#define DAY_HISTORY_LENGTH 31

typedef struct {
	VeTinyItem daySequenceNumber;
#if CFG_VEDIRECT_WITH_HISTORY || CFG_VEDIRECT_WITH_HISTORY_TOTAL
	VeTinyItem totals;
#endif
#if CFG_VEDIRECT_WITH_HISTORY
	VeTinyItem days[DAY_HISTORY_LENGTH];
#elif CFG_VEDIRECT_WITH_HISTORY_DAY0
	VeTinyItem days[1];
#endif
} MpptHistory;

typedef struct {
	VeTinyItem batteryBulkTimeLimit;
	VeTinyItem batteryAbsorptionTimeLimit;
	VeTinyItem batteryAbsorptionVoltage;
	VeTinyItem batteryFloatVoltage;
	VeTinyItem batteryTempCompensation;
	VeTinyItem batteryType;
	VeTinyItem batteryMaxCurrent;
	VeTinyItem batteryVoltage;
	VeTinyItem batteryVoltageSetting;
#if !CFG_VEDIRECT_WITHOUT_MPPT_NETWORKING
	VeTinyItem batteryBmsPresent;
#endif
	VeTinyItem dcOutputControl;
	VeTinyItem dcOutputSwitchHighLevel;
	VeTinyItem dcOutputSwitchLowLevel;
} MpptSettings;

typedef struct {
	VeTinyItem batteryMaxCurrrent;
	VeTinyItem batteryVoltageRange;
	VeTinyItem historyVersion;
	VeTinyItem streetlightVersion;
} MpptCapabilities;

typedef struct {
	VeTinyItem networkMode;
	VeTinyItem networkStatus;
	VeTinyItem chargeVoltageSetpoint;
	VeTinyItem chargeCurrentPercentage;
	VeTinyItem chargeCurrentLimit;
} MpptLinkItems;

typedef struct
{
	MpptPanel		 panel;
	MpptCharger		 charger;
	MpptLoad		 load;
	MpptState		 state;
	MpptSettings	 settings;
	MpptCapabilities capabilities;
#if !CFG_VEDIRECT_WITHOUT_MPPT_NETWORKING
	MpptLinkItems    link;
#endif
#if CFG_VEDIRECT_WITH_HISTORY || CFG_VEDIRECT_WITH_HISTORY_DAY0 || CFG_VEDIRECT_WITH_HISTORY_TOTAL
	MpptHistory		 history;
#endif
} VeMppt;

#endif // _VEDIRECT_MPPT_H_
