#include <velib/base/base.h>
#include <velib/utils/ve_logger.h>

#include <ctype.h>
#include <string.h>
#include "vetext_fh.h"
#include "vetext.h"
#include "vehex.h"

#define MODULE "VE.Frame"

// The name of the record that contains the checksum.
#define RECORD_NAME_CHECKSUM			"CHECKSUM"

void vhInit(VdTextFhInstance * instance)
{
	memset(instance, 0, sizeof(*instance));
	instance->invalidFrame = veTrue;
}

void vhRxData(VdTextFhInstance * instance, VdTextInstance * textInstance, VdHexInstance * hexInstance, un8 inbyte)
{
	instance->checksum += inbyte;
	inbyte = toupper(inbyte);
	switch(instance->rxState)
	{
	case IDLE:
		/* wait for \n of the start of an record */
		switch(inbyte)
		{
		case '\n':
			instance->rxState = RECORD_BEGIN;
			break;
		case '\r': /* Preset checksum */
			instance->checksum = '\r';
			break;
		case ':': /* HEX protocol*/
			instance->rxState = RECORD_HEX;
			break;
		default:
			break;
		}
		break;
	case RECORD_BEGIN:
		switch (inbyte)
		{
		case ':': /* HEX protocol*/
			instance->rxState = RECORD_HEX;
			break;
		default: /* TEXT Protocol */
			instance->pName = instance->name;
			*(instance->pName)++ = inbyte;
			instance->rxState = RECORD_NAME;
			break;
		}
		break;
	case RECORD_NAME:
		// The record name is being received, terminated by a \t
		switch(inbyte)
		{
		case '\t':
			instance->pValue = instance->value; /* Reset value pointer */
			// the Checksum record indicates a EOR
			if ( instance->pName < (instance->name + sizeof(instance->name)) ) {
				*(instance->pName) = 0; /* Zero terminate */
				if (strcmp(instance->name, RECORD_NAME_CHECKSUM) == 0) {
					instance->rxState = CHECKSUM;
					break;
				}
			}
			// else
			//	not treated as an error since future record might be longer,
			//	only use the first VE_TEXT_MAX_NAME_LENGTH characters
			// Mmmmmm I (MMU) think it should error here.......
			instance->rxState = RECORD_VALUE;
			break;
		case ':':
			instance->rxState = RECORD_HEX;
			break;
		default:
			// add byte to name, but do no overflow
			if ( instance->pName < (instance->name + sizeof(instance->name)) )
				*(instance->pName)++ = inbyte;
			break;
		}
		break;
	case RECORD_VALUE:
		// The record value is being received.  The \r indicates a new record.
		switch(inbyte)
		{
		case '\n':
			// forward record, only if it could be stored completely
			if ( instance->pValue < (instance->value + sizeof(instance->value))) {
				*(instance->pValue) = 0; // make zero ended
				if (!instance->invalidFrame) veTextRxEvent(textInstance, instance->name, instance->value);
			}
			instance->rxState = RECORD_BEGIN;
			break;
		case '\r': /* Skip */
			break;
		case ':':
			instance->rxState = RECORD_HEX;
			break;
		default:
			// add byte to value, but do no overflow
			if ( instance->pValue < (instance->value + sizeof(instance->value)) )
				*(instance->pValue)++ = inbyte;
			break;
		}
		break;
	case CHECKSUM:
		/* Added mask for XAP (CSR) platform because un8 is 16 bit on XAP */
		if ((instance->checksum & 0xFF) == 0) {
			instance->invalidFrame = veFalse;
			veTextFrameEndEvent(textInstance, veTrue);
		} else {
			instance->checksum = 0;
			instance->invalidFrame = veTrue;
		}
		instance->rxState = IDLE;
		break;

	case RECORD_HEX:
		if (veHexRxEvent(hexInstance, inbyte))
			instance->rxState = IDLE;
		break;
	}
}
