#include <stdlib.h>
#include <string.h>
#include <velib/utils/ve_assert.h>
#include "veti.h"

/*
 * Defines
 */

/*
 * Locals
 */

/*
 * Local functions prototype
 */
static void vetiInformSubscribers(VetiInstance * instance, SubHandles handle, const VetiId id, VeTinyItem const * const item);
static void vetiInformSubscriber(VetiInstance * instance, SubHandles handle, const VetiId id, VeTinyItem const * const item);
static veBool vetiSendPublishRequest(VetiInstance * instance, VetiId id);
static veBool vetiChangeSubscription(VetiInstance * instance, SubHandles handle, VetiId id, veBool subscribe);

/*
 * Local functions
 */
static inline void vetiInformSubscriber(VetiInstance * instance, SubHandles handle, const VetiId id, VeTinyItem const * const item)
{
	switch (handle) {
	case subHandle1:
		if (instance->subReplyFunc[0] && (item->properties.subscriber1 || instance->subToAllItems.subscriber1))
			instance->subReplyFunc[0](instance->subscriberContext[0], id, &item->value, item->properties.valid);
		break;
	case subHandle2:
		if (instance->subReplyFunc[1] && (item->properties.subscriber2 || instance->subToAllItems.subscriber2))
			instance->subReplyFunc[1](instance->subscriberContext[1], id, &item->value, item->properties.valid);
		break;
	case subHandle3:
		if (instance->subReplyFunc[2] && (item->properties.subscriber3 || instance->subToAllItems.subscriber3))
			instance->subReplyFunc[2](instance->subscriberContext[2], id, &item->value, item->properties.valid);
		break;
	case noSubHandle:
		break;
	}
}

static void vetiInformSubscribers(VetiInstance * instance, SubHandles handle, const VetiId id, VeTinyItem const * const item)
{
	if (handle != subHandle1) vetiInformSubscriber(instance, subHandle1,id,item);
	if (handle != subHandle2) vetiInformSubscriber(instance, subHandle2,id,item);
	if (handle != subHandle3) vetiInformSubscriber(instance, subHandle3,id,item);
}

static veBool vetiSendPublishRequest(VetiInstance * instance, VetiId id)
{
	veBool requestSucces = veFalse;
	if (instance->pubReqFunc) {
		instance->pubReqFunc(instance->publisherContext, id);
		requestSucces = veTrue;
	}
	return requestSucces;
}

static veBool vetiChangeSubscription(VetiInstance * instance, SubHandles handle, VetiId id, veBool subscribe)
{
	VeTinyItem * const item = vetiItemFind(&instance->veItems.items, id);
	if (item == NULL) return veFalse;
	switch (handle) {
	case subHandle1:
		item->properties.subscriber1 = subscribe;
		break;
	case subHandle2:
		item->properties.subscriber2 = subscribe;
		break;
	case subHandle3:
		item->properties.subscriber3 = subscribe;
		break;
	default:
		return veFalse;
	}
	if (subscribe)
		if (item->properties.request)
			vetiSendPublishRequest(instance, id);
	return veTrue;
}

/*
 * API functions
 */

void vetiInit(VetiInstance * instance)
{
	memset(instance, 0, sizeof(*instance));
}

SubHandles vetiConnect(VetiInstance * instance)
{
	if (instance->subscribers.subscriber1)
		if (instance->subscribers.subscriber2)
			if (instance->subscribers.subscriber3)
				return noSubHandle;
			else
				return subHandle3;
		else
			return subHandle2;
	else
		return subHandle1;
}

veBool vetiConnectSubscriber(VetiInstance * instance, const SubHandles handle, SubReply replyFunc, void * const userContext)
{
	switch (handle) {
	case subHandle1:
		instance->subscribers.subscriber1 = veTrue;
		break;
	case subHandle2:
		instance->subscribers.subscriber2 = veTrue;
		break;
	case subHandle3:
		instance->subscribers.subscriber3 = veTrue;
		break;
	default:
		return veFalse;
	}
	instance->subReplyFunc[handle] = replyFunc;
	instance->subscriberContext[handle] = userContext;

	return veTrue;
}

void vetiConnectPublischer(VetiInstance * instance, const PubReqFunc reqFunc, void * userContext)
{
	instance->pubReqFunc = reqFunc;
	instance->publisherContext = userContext;
}

/* Publisching means publishing, eventough the value is the same */
veBool vetiPublish(VetiInstance * instance, const SubHandles handle, const VetiId id, const VeVariantValue * value)
{
	VeTinyItem * const item = vetiItemFind(&instance->veItems.items, id);
	if (item == NULL) return veFalse;
	item->value = *value;
	item->properties.valid = veTrue;
	vetiPublishItem(instance, handle, id, item);
	return veTrue;
}

void vetiPublishItem(VetiInstance * instance, const SubHandles handle, const VetiId id, VeTinyItem *item)
{
	vetiInformSubscribers(instance, handle, id, item);
}

veBool vetiPublishRequest(VetiInstance * instance, const SubHandles handle, VetiId id)
{
	VeTinyItem * const item = vetiItemFind(&instance->veItems.items, id);
	if (item == NULL) {
#if CFG_VEDIRECT_LAST_MESSAGE
		vetiSendPublishRequest(instance, id);
		return veTrue;
#else
		return veFalse;
#endif
	}
	if (item->properties.request) {
		/* Always invalidate when the request flag is set, so the application knows when it has been received again */
		item->properties.valid = veFalse;
		return vetiSendPublishRequest(instance, id);
	} else if (item->properties.valid) {
		vetiInformSubscriber(instance, handle, id, item);
		return veTrue;
	} else
		return vetiSendPublishRequest(instance, id);
}

veBool vetiInvalidateById(VetiInstance * instance, const SubHandles handle, VetiId id)
{
	VE_UNUSED(handle);
	VeTinyItem * const item = vetiItemFind(&instance->veItems.items, id);
	if (item == NULL) return veFalse;
	item->properties.valid = veFalse;
	vetiInformSubscribers(instance, handle, id, item);
	return veTrue;
}

veBool vetiSubscribe(VetiInstance * instance, const SubHandles handle, const VetiId id)
{
	return vetiChangeSubscription(instance, handle, id, veTrue);
}

void vetiSubscribeAll(VetiInstance * instance, const SubHandles handle)
{
	switch (handle) {
	case subHandle1:
		instance->subToAllItems.subscriber1 = veTrue;
		break;
	case subHandle2:
		instance->subToAllItems.subscriber2 = veTrue;
		break;
	case subHandle3:
		instance->subToAllItems.subscriber3 = veTrue;
		break;
	case noSubHandle:
		break;
	}
}

veBool vetiUnSubscribe(VetiInstance * instance, const SubHandles handle, const VetiId id)
{
	return vetiChangeSubscription(instance, handle, id, veFalse);
}

void vetiUnSubscribeAll(VetiInstance * instance, const SubHandles handle)
{
	VeTinyItem * item;
	VeTinyItem * const begin = vetiItemBegin(&instance->veItems.items);
	VeTinyItem * const end = vetiItemEnd(&instance->veItems.items);
	switch (handle) {
	case subHandle1:
		instance->subToAllItems.subscriber1 = veFalse;
		for (item = begin; item < end; item=vetiItemNext(item))
			item->properties.subscriber1 = veFalse;
		break;
	case subHandle2:
		instance->subToAllItems.subscriber2 = veFalse;
		for (item = begin; item < end; item=vetiItemNext(item))
			item->properties.subscriber2 = veFalse;
		break;
	case subHandle3:
		instance->subToAllItems.subscriber3 = veFalse;
		for (item = begin; item < end; item=vetiItemNext(item))
			item->properties.subscriber3 = veFalse;
		break;
	case noSubHandle:
		break;
	}
}

veBool vetiGetValue(VetiInstance * instance, const VetiId id, VeVariantValue * const value)
{
	const VeTinyItem * const item = vetiItemFind(&instance->veItems.items, id);
	if ( (item == NULL) || (item->properties.valid == veFalse) ) return veFalse;
	*value = item->value;
	return veTrue;
}

veBool vetiIsValid(VetiInstance * instance, const VetiId id)
{
	const VeTinyItem * const item = vetiItemFind(&instance->veItems.items, id);
	if ( (item == NULL) || (item->properties.valid == veFalse) )
		return veFalse;
	else
		return veTrue;
}

