#include <stdlib.h>
#include <string.h>

#include <velib/ve_regs.h>
#include <velib/utils/ve_assert.h>
#include <velib/utils/ve_logger.h>

#include "veti_items.h"

#define MODULE "VETI.Items"

typedef struct {
	VetiId const id;
	size_t offset;
} RegIdIndex;

/* This index is sorted on RegId */
const RegIdIndex itemIndex[] = {
	{ VE_REG_RESTORE_DEFAULT, offsetof(VetiItems, device.factoryDefault) },
	{ VE_REG_PRODUCT_ID, offsetof(VetiItems, device.prodId) },
	{ VE_REG_APP_VER, offsetof(VetiItems, device.firmwareVersion) },
	{ VE_REG_SERIAL_NUMBER, offsetof(VetiItems, device.serial) },
	{ VE_REG_CAPABILITIES1, offsetof(VetiItems, device.capabilities1) },
#if CFG_VETEXT_WITH_INVERTER
	{ VE_REG_DEVICE_MODE, offsetof(VetiItems, device.mode) },
#endif
#if CFG_VETEXT_WITH_MPPT || CFG_VETEXT_WITH_INVERTER
	{ VE_REG_DEVICE_STATE, offsetof(VetiItems, device.state) },
#endif
#if CFG_VETEXT_WITH_BMV && CFG_VEDIRECT_WITH_HISTORY
	{ VE_REG_HIST_DEEPEST_DISCHARGE, offsetof(VetiItems, product.bmv.history.deepestDischarge) },
	{ VE_REG_HIST_LAST_DISCHARGE, offsetof(VetiItems, product.bmv.history.lastDischarge) },
	{ VE_REG_HIST_AVERAGE_DISCHARGE, offsetof(VetiItems, product.bmv.history.averageDischarge) },
	{ VE_REG_HIST_NR_OF_CHARGE_CYCLES, offsetof(VetiItems, product.bmv.history.chargeCycles) },
	{ VE_REG_HIST_NR_OF_FULL_DISCHARGES, offsetof(VetiItems, product.bmv.history.averageDischarge) },
	{ VE_REG_HIST_CUMULATIVE_AH, offsetof(VetiItems, product.bmv.history.totalAhDrawn) },
	{ VE_REG_HIST_MIN_VOLTAGE, offsetof(VetiItems, product.bmv.history.minimumVoltage) },
	{ VE_REG_HIST_MAX_VOLTAGE, offsetof(VetiItems, product.bmv.history.maximumVoltage) },
	{ VE_REG_HIST_SECS_SINCE_LAST_FULL_CHARGE, offsetof(VetiItems, product.bmv.history.timeSinceLastFullCharge) },
	{ VE_REG_HIST_NR_OF_AUTO_SYNCS, offsetof(VetiItems, product.bmv.history.automaticSyncs) },
	{ VE_REG_HIST_NR_OF_LOW_VOLTAGE_ALARMS, offsetof(VetiItems, product.bmv.history.lowStarterVoltageAlarms) },
	{ VE_REG_HIST_NR_OF_HIGH_VOLTAGE_ALARMS, offsetof(VetiItems, product.bmv.history.highStarterVoltageAlarms) },
	{ VE_REG_HIST_NR_OF_LOW_VOLTAGE_2_ALARMS, offsetof(VetiItems, product.bmv.history.lowVoltageAlarms) },
	{ VE_REG_HIST_NR_OF_HIGH_VOLTAGE_2_ALARMS, offsetof(VetiItems, product.bmv.history.highVoltageAlarms) },
	{ VE_REG_HIST_MIN_VOLTAGE_2, offsetof(VetiItems, product.bmv.history.minimumStarterVoltage) },
	{ VE_REG_HIST_MAX_VOLTAGE_2, offsetof(VetiItems, product.bmv.history.maximumStarterVoltage) },
	{ VE_REG_HIST_KWH_OUT, offsetof(VetiItems, product.bmv.history.dischargedEnergy) },
	{ VE_REG_HIST_KWH_IN, offsetof(VetiItems, product.bmv.history.chargedEnergy) },
#endif
#if CFG_VETEXT_WITH_INVERTER
	{ VE_REG_WARNING_REASON, offsetof(VetiItems, device.warningReason) },
#endif
#if CFG_VETEXT_WITH_BMV || CFG_VETEXT_WITH_INVERTER
	{ VE_REG_ALARM_REASON, offsetof(VetiItems, device.alarmReason) },
#endif
#if CFG_VETEXT_WITH_BMV || CFG_VETEXT_WITH_MPPT
	{ VE_REG_RELAY_CONTROL, offsetof(VetiItems, device.relayState) },
#endif
#if CFG_VETEXT_WITH_BMV
	{ VE_REG_BATTERY_MID_POINT_VOLTAGE, offsetof(VetiItems, battery.midVoltage) },
	{ VE_REG_BATTERY_MID_POINT_DEVIATION, offsetof(VetiItems, battery.midVoltageDev) },
	{ VE_REG_TTG, offsetof(VetiItems, product.bmv.state.timeToGo) },
	{ VE_REG_SOC, offsetof(VetiItems, product.bmv.state.stateOfCharge) },
#endif
	{ VE_REG_CLEAR_HISTORY, offsetof(VetiItems, device.clearHistory) },
#if CFG_VETEXT_WITH_MPPT
#if	CFG_VEDIRECT_WITH_HISTORY || CFG_VEDIRECT_WITH_HISTORY_TOTAL
	{ VE_REG_HISTORY_TOTAL, offsetof(VetiItems, product.mppt.history.totals) },
#endif
#if CFG_VEDIRECT_WITH_HISTORY || CFG_VEDIRECT_WITH_HISTORY_DAY0
	{ VE_REG_HISTORY_DAY00, offsetof(VetiItems, product.mppt.history.days[0]) },
#endif
#if CFG_VEDIRECT_WITH_HISTORY
	{ VE_REG_HISTORY_DAY01,	offsetof(VetiItems, product.mppt.history.days[1]) },
	{ VE_REG_HISTORY_DAY02,	offsetof(VetiItems, product.mppt.history.days[2]) },
	{ VE_REG_HISTORY_DAY03,	offsetof(VetiItems, product.mppt.history.days[3]) },
	{ VE_REG_HISTORY_DAY04,	offsetof(VetiItems, product.mppt.history.days[4]) },
	{ VE_REG_HISTORY_DAY05,	offsetof(VetiItems, product.mppt.history.days[5]) },
	{ VE_REG_HISTORY_DAY06,	offsetof(VetiItems, product.mppt.history.days[6]) },
	{ VE_REG_HISTORY_DAY07,	offsetof(VetiItems, product.mppt.history.days[7]) },
	{ VE_REG_HISTORY_DAY08,	offsetof(VetiItems, product.mppt.history.days[8]) },
	{ VE_REG_HISTORY_DAY09,	offsetof(VetiItems, product.mppt.history.days[9]) },
	{ VE_REG_HISTORY_DAY10,	offsetof(VetiItems, product.mppt.history.days[10]) },
	{ VE_REG_HISTORY_DAY11,	offsetof(VetiItems, product.mppt.history.days[11]) },
	{ VE_REG_HISTORY_DAY12,	offsetof(VetiItems, product.mppt.history.days[12]) },
	{ VE_REG_HISTORY_DAY13,	offsetof(VetiItems, product.mppt.history.days[13]) },
	{ VE_REG_HISTORY_DAY14,	offsetof(VetiItems, product.mppt.history.days[14]) },
	{ VE_REG_HISTORY_DAY15,	offsetof(VetiItems, product.mppt.history.days[15]) },
	{ VE_REG_HISTORY_DAY16,	offsetof(VetiItems, product.mppt.history.days[16]) },
	{ VE_REG_HISTORY_DAY17,	offsetof(VetiItems, product.mppt.history.days[17]) },
	{ VE_REG_HISTORY_DAY18,	offsetof(VetiItems, product.mppt.history.days[18]) },
	{ VE_REG_HISTORY_DAY19,	offsetof(VetiItems, product.mppt.history.days[19]) },
	{ VE_REG_HISTORY_DAY20,	offsetof(VetiItems, product.mppt.history.days[20]) },
	{ VE_REG_HISTORY_DAY21,	offsetof(VetiItems, product.mppt.history.days[21]) },
	{ VE_REG_HISTORY_DAY22,	offsetof(VetiItems, product.mppt.history.days[22]) },
	{ VE_REG_HISTORY_DAY23,	offsetof(VetiItems, product.mppt.history.days[23]) },
	{ VE_REG_HISTORY_DAY24,	offsetof(VetiItems, product.mppt.history.days[24]) },
	{ VE_REG_HISTORY_DAY25,	offsetof(VetiItems, product.mppt.history.days[25]) },
	{ VE_REG_HISTORY_DAY26,	offsetof(VetiItems, product.mppt.history.days[26]) },
	{ VE_REG_HISTORY_DAY27,	offsetof(VetiItems, product.mppt.history.days[27]) },
	{ VE_REG_HISTORY_DAY28,	offsetof(VetiItems, product.mppt.history.days[28]) },
	{ VE_REG_HISTORY_DAY29,	offsetof(VetiItems, product.mppt.history.days[29]) },
	{ VE_REG_HISTORY_DAY30,	offsetof(VetiItems, product.mppt.history.days[30]) },
#endif
#if !CFG_VEDIRECT_WITHOUT_MPPT_NETWORKING
	{ VE_REG_LINK_VSET, offsetof(VetiItems, product.mppt.link.chargeVoltageSetpoint) },
	{ VE_REG_LINK_NETWORK_MODE, offsetof(VetiItems, product.mppt.link.networkMode) },
	{ VE_REG_LINK_NETWORK_STATUS, offsetof(VetiItems, product.mppt.link.networkStatus) },
	{ VE_REG_LINK_CHARGE_CURRENT_PERCENTAGE, offsetof(VetiItems, product.mppt.link.chargeCurrentPercentage) },
	{ VE_REG_LINK_CHARGE_CURRENT_LIMIT, offsetof(VetiItems, product.mppt.link.chargeCurrentLimit) },
#endif
#endif
#if CFG_VETEXT_WITH_INVERTER
	{ VE_REG_AC_OUT_VOLTAGE, offsetof(VetiItems, ac.voltage) },
	{ VE_REG_AC_OUT_CURRENT, offsetof(VetiItems, ac.current) },
#endif
#if CFG_VEDIRECT_LAST_MESSAGE
	{ VE_REG_LAST_MESSAGE, offsetof(VetiItems, device.lastMessage) },
#endif
#if CFG_VETEXT_WITH_MPPT && (CFG_VEDIRECT_WITH_HISTORY || CFG_VEDIRECT_WITH_HISTORY_DAY0 || CFG_VEDIRECT_WITH_HISTORY_TOTAL)
	{ VE_REG_HSDS, offsetof(VetiItems, product.mppt.history.daySequenceNumber) },
#endif
#if CFG_VETEXT_WITH_BMV
	{ VE_REG_DC_CHANNEL2_VOLTAGE, offsetof(VetiItems, product.bmv.auxiliary.voltage) },
#endif
	{ VE_REG_DC_CHANNEL1_VOLTAGE, offsetof(VetiItems, battery.voltage) },
#if CFG_VETEXT_WITH_BMV || CFG_VETEXT_WITH_MPPT
	{ VE_REG_DC_CHANNEL1_POWER, offsetof(VetiItems, battery.power) },
	{ VE_REG_DC_CHANNEL1_CURRENT, offsetof(VetiItems, battery.current) },
#endif
#if CFG_VETEXT_WITH_MPPT
	{ VE_REG_DC_OUTPUT_SWITCH_LOW_LEVEL, offsetof(VetiItems, product.mppt.settings.dcOutputSwitchLowLevel) },
	{ VE_REG_DC_OUTPUT_SWITCH_HIGH_LEVEL, offsetof(VetiItems, product.mppt.settings.dcOutputSwitchHighLevel) },
	{ VE_REG_DC_OUTPUT_STATUS, offsetof(VetiItems, product.mppt.state.load) },
	{ VE_REG_DC_OUTPUT_VOLTAGE, offsetof(VetiItems, product.mppt.load.voltage) },
	{ VE_REG_DC_OUTPUT_POWER, offsetof(VetiItems, product.mppt.load.power) },
	{ VE_REG_DC_OUTPUT_CONTROL, offsetof(VetiItems, product.mppt.settings.dcOutputControl) },
	{ VE_REG_DC_OUTPUT_CURRENT, offsetof(VetiItems, product.mppt.load.current) },
	{ VE_REG_DC_INPUT_VOLTAGE, offsetof(VetiItems, product.mppt.panel.voltage) },
	{ VE_REG_DC_INPUT_POWER, offsetof(VetiItems, product.mppt.panel.power) },
	{ VE_REG_DC_INPUT_CURRENT, offsetof(VetiItems, product.mppt.panel.current) },
	{ VE_REG_CHR_STREETLIGHT_VERSION, offsetof(VetiItems, product.mppt.capabilities.streetlightVersion) },
	{ VE_REG_CHR_HISTORY_VERSION, offsetof(VetiItems, product.mppt.capabilities.historyVersion) },
	{ VE_REG_CHR_VOLTAGE_SETTINGS_RANGE, offsetof(VetiItems, product.mppt.capabilities.batteryVoltageRange) },
	{ VE_REG_CHR_YESTERDAY_PMAX, offsetof(VetiItems, product.mppt.charger.yesterdayPmax) },
	{ VE_REG_CHR_YESTERDAY_YIELD, offsetof(VetiItems, product.mppt.charger.yesterdayYield) },
	{ VE_REG_CHR_TODAY_PMAX, offsetof(VetiItems, product.mppt.charger.todayPmax) },
	{ VE_REG_CHR_TODAY_YIELD, offsetof(VetiItems, product.mppt.charger.todayYield) },
	{ VE_REG_CHR_VOLTAGE, offsetof(VetiItems, product.mppt.charger.voltage) },
	{ VE_REG_CHR_POWER, offsetof(VetiItems, product.mppt.charger.power) },
	{ VE_REG_CHR_CURRENT, offsetof(VetiItems, product.mppt.charger.current) },
	{ VE_REG_CHR_ERROR_CODE, offsetof(VetiItems, product.mppt.state.error) },
	{ VE_REG_CHR_USER_YIELD, offsetof(VetiItems, product.mppt.charger.userYield) },
	{ VE_REG_CHR_MAX_CURRENT, offsetof(VetiItems, product.mppt.capabilities.batteryMaxCurrrent) },
#if !CFG_VEDIRECT_WITHOUT_MPPT_NETWORKING
	{ VE_REG_BAT_BMS_PRESENT, offsetof(VetiItems, product.mppt.settings.batteryBmsPresent) },
#endif
	{ VE_REG_BAT_VOLTAGE_SETTING, offsetof(VetiItems, product.mppt.settings.batteryVoltageSetting) },
#endif
#if CFG_VETEXT_WITH_BMV
	{ VE_REG_BAT_TEMPERATURE, offsetof(VetiItems, battery.temperature) },
#endif
#if CFG_VETEXT_WITH_MPPT
	{ VE_REG_BAT_VOLTAGE, offsetof(VetiItems, product.mppt.settings.batteryVoltage) },
	{ VE_REG_BAT_MAX_CURRENT, offsetof(VetiItems, product.mppt.settings.batteryMaxCurrent) },
	{ VE_REG_BAT_TYPE, offsetof(VetiItems, product.mppt.settings.batteryType) },
	{ VE_REG_BAT_TEMP_COMP, offsetof(VetiItems, product.mppt.settings.batteryTempCompensation) },
	{ VE_REG_BAT_FLOAT_V, offsetof(VetiItems, product.mppt.settings.batteryFloatVoltage) },
	{ VE_REG_BAT_ABSORPTION_V, offsetof(VetiItems, product.mppt.settings.batteryAbsorptionVoltage) },
	{ VE_REG_BAT_ABS_T_LIMIT, offsetof(VetiItems, product.mppt.settings.batteryAbsorptionTimeLimit) },
	{ VE_REG_BAT_BULK_T_LIMIT, offsetof(VetiItems, product.mppt.settings.batteryBulkTimeLimit) },
#endif
#if CFG_VETEXT_WITH_BMV
	{ VE_REG_CAH, offsetof(VetiItems, product.bmv.state.consumedAmphours) },
#endif
};
const un8 veDirectItemCount = sizeof(itemIndex)/sizeof(RegIdIndex);

/*
 * Local functions prototype
 */
static int searchCompare(const void * a, const void * b);

/*
 * Local functions
 */
static int searchCompare(const void * a, const void * b)
{
	const un16 key = *(const un16 *)a;
	const un16 elem = ((const RegIdIndex *)b)->id;
	if (key < elem)	return -1;
	else if (key > elem) return 1;
	else return 0;
}

/*
 * API functions
 */
void vetiItemInit(VetiData * vetiData, const VeProdClasses productClass)
{
	VetiItems * const items = &vetiData->items;

#if CFG_ASSERT_ENABLED
{
	un8 i;
	for (i = 0; i < veDirectItemCount-1; i++) {
		if (!(itemIndex[i].id < itemIndex[i+1].id)) {
			logE(MODULE, "Check failed at item index %u: %4.4X %4.4X", i, itemIndex[i].id, itemIndex[i+1].id);
			veAssert(itemIndex[i].id < itemIndex[i+1].id);
		}
	}
}
#endif

	memset(&vetiData->items, 0, sizeof(VetiItems));

	VeTinyItem * item = vetiItemFind(items, VE_REG_SERIAL_NUMBER);
	item->value.Ptr = vetiData->ser;
	item->properties.type = VETI_PTR;

#if CFG_VEDIRECT_WITH_HISTORY || CFG_VEDIRECT_WITH_HISTORY_TOTAL
{
	VeTinyItem * item = vetiItemFind(items, VE_REG_HISTORY_TOTAL);
	item->value.Ptr = &vetiData->totalsHistory;
	item->properties.type = VETI_PTR;
}
#endif
#if CFG_VEDIRECT_WITH_HISTORY
{
	un8 i;
	VeTinyItem * item = vetiItemFind(items, VE_REG_HISTORY_DAY00);
	MpptDayHistory * day = vetiData->daysHistory;
	for (i = 0; i < DAY_HISTORY_LENGTH; i++) {
		item->properties.type = VETI_PTR;
		item->value.Ptr = day;
		item++;
		day++;
	}
}
#elif CFG_VEDIRECT_WITH_HISTORY_DAY0
{
	VeTinyItem * item = vetiItemFind(items, VE_REG_HISTORY_DAY00);
	item->value.Ptr = &vetiData->daysHistory[0];
	item->properties.type = VETI_PTR;
}
#endif
#if CFG_VEDIRECT_LAST_MESSAGE
{
	VeTinyItem * item = vetiItemFind (items, VE_REG_LAST_MESSAGE);
	item->value.Ptr = &vetiData->lastMessage;
	item->properties.type = VETI_PTR;
}
#endif

	items->device.prodClass.properties.type			= VETI_UN16;
	items->device.prodId.properties.type			= VETI_UN16;
	items->device.firmwareVersion.properties.type	= VETI_UN16;
	items->device.clearHistory.properties.type		= VETI_UN8;
	items->device.factoryDefault.properties.type	= VETI_UN8;
	items->device.capabilities1.properties.type		= VETI_UN32;
	items->battery.voltage.properties.type			= VETI_UN16;
#if CFG_VETEXT_WITH_BMV || CFG_VETEXT_WITH_MPPT
	items->battery.current.properties.type			= VETI_SN16;
	items->battery.power.properties.type			= VETI_SN16;
	items->device.relayState.properties.type		= VETI_UN16;
#endif
#if CFG_VETEXT_WITH_BMV
	items->battery.temperature.properties.type		= VETI_UN16;
	items->battery.midVoltage.properties.type		= VETI_UN16;
	items->battery.midVoltageDev.properties.type	= VETI_SN16;
#endif
#if CFG_VETEXT_WITH_INVERTER
	items->ac.voltage.properties.type				= VETI_UN16;
	items->ac.current.properties.type				= VETI_SN16;
#endif
#if CFG_VETEXT_WITH_BMV || CFG_VETEXT_WITH_INVERTER
	items->device.alarmReason.properties.type		= VETI_UN16;
#endif
#if CFG_VETEXT_WITH_INVERTER
	items->device.warningReason.properties.type		= VETI_UN16;
#endif
#if CFG_VETEXT_WITH_MPPT || CFG_VETEXT_WITH_INVERTER
	items->device.state.properties.type				= VETI_UN8;
#endif
#if CFG_VETEXT_WITH_INVERTER
	items->device.mode.properties.type				= VETI_UN8;
#endif
	switch (productClass) {
#if CFG_VETEXT_WITH_BMV
	case classBattery:
		items->product.bmv.state.consumedAmphours.properties.type			= VETI_SN32;
		items->product.bmv.state.stateOfCharge.properties.type				= VETI_UN16;
		items->product.bmv.state.timeToGo.properties.type					= VETI_UN16;
		items->product.bmv.state.temperature.properties.type				= VETI_UN16;
		items->product.bmv.auxiliary.voltage.properties.type				= VETI_UN16;
#if CFG_VEDIRECT_WITH_HISTORY
		items->product.bmv.history.deepestDischarge.properties.type			= VETI_SN32;
		items->product.bmv.history.lastDischarge.properties.type			= VETI_SN32;
		items->product.bmv.history.averageDischarge.properties.type			= VETI_SN32;
		items->product.bmv.history.chargeCycles.properties.type				= VETI_UN32;
		items->product.bmv.history.fullDischarges.properties.type			= VETI_UN32;
		items->product.bmv.history.totalAhDrawn.properties.type				= VETI_SN32;
		items->product.bmv.history.minimumVoltage.properties.type			= VETI_SN32;
		items->product.bmv.history.maximumVoltage.properties.type			= VETI_SN32;
		items->product.bmv.history.timeSinceLastFullCharge.properties.type	= VETI_UN32;
		items->product.bmv.history.automaticSyncs.properties.type			= VETI_UN32;
		items->product.bmv.history.lowStarterVoltageAlarms.properties.type	= VETI_UN32;
		items->product.bmv.history.highStarterVoltageAlarms.properties.type = VETI_UN32;
		items->product.bmv.history.minimumStarterVoltage.properties.type	= VETI_SN32;
		items->product.bmv.history.maximumStarterVoltage.properties.type	= VETI_SN32;
		items->product.bmv.history.chargedEnergy.properties.type			= VETI_UN32;
		items->product.bmv.history.dischargedEnergy.properties.type			= VETI_UN32;
		items->product.bmv.history.lowVoltageAlarms.properties.type			= VETI_UN32;
		items->product.bmv.history.highVoltageAlarms.properties.type		= VETI_UN32;
#endif
		break;
#endif
#if CFG_VETEXT_WITH_MPPT
	case classSolarCharger:
		items->product.mppt.panel.voltage.properties.type			= VETI_UN16;
		items->product.mppt.panel.current.properties.type			= VETI_UN16;
		items->product.mppt.panel.power.properties.type				= VETI_UN32;
		items->product.mppt.charger.voltage.properties.type			= VETI_UN16;
		items->product.mppt.charger.current.properties.type			= VETI_SN16;
		items->product.mppt.charger.power.properties.type			= VETI_UN16;
		items->product.mppt.charger.userYield.properties.type		= VETI_UN32;
		items->product.mppt.charger.todayPmax.properties.type		= VETI_UN32;
		items->product.mppt.charger.todayYield.properties.type		= VETI_UN32;
		items->product.mppt.charger.yesterdayPmax.properties.type	= VETI_UN32;
		items->product.mppt.charger.yesterdayYield.properties.type	= VETI_UN32;
		items->product.mppt.load.voltage.properties.type			= VETI_UN16;
		items->product.mppt.load.current.properties.type			= VETI_UN16;
		items->product.mppt.load.power.properties.type				= VETI_UN16;
		items->product.mppt.state.error.properties.type				= VETI_UN8;
		items->product.mppt.state.load.properties.type				= VETI_UN8;
		items->product.mppt.settings.batteryBulkTimeLimit.properties.type			= VETI_UN16;
		items->product.mppt.settings.batteryAbsorptionTimeLimit.properties.type		= VETI_UN16;
		items->product.mppt.settings.batteryAbsorptionVoltage.properties.type		= VETI_UN16;
		items->product.mppt.settings.batteryFloatVoltage.properties.type			= VETI_UN16;
		items->product.mppt.settings.batteryTempCompensation.properties.type		= VETI_UN16;
		items->product.mppt.settings.batteryType.properties.type					= VETI_UN8;
		items->product.mppt.settings.batteryMaxCurrent.properties.type				= VETI_UN16;
		items->product.mppt.settings.batteryVoltage.properties.type					= VETI_UN8;
		items->product.mppt.settings.batteryVoltageSetting.properties.type			= VETI_UN8;
#if !CFG_VEDIRECT_WITHOUT_MPPT_NETWORKING
		items->product.mppt.settings.batteryBmsPresent.properties.type				= VETI_UN8;
#endif
		items->product.mppt.settings.dcOutputControl.properties.type				= VETI_UN8;
		items->product.mppt.settings.dcOutputSwitchHighLevel.properties.type		= VETI_UN16;
		items->product.mppt.settings.dcOutputSwitchLowLevel.properties.type			= VETI_UN16;
		items->product.mppt.capabilities.batteryMaxCurrrent.properties.type			= VETI_UN16;
		items->product.mppt.capabilities.batteryVoltageRange.properties.type		= VETI_UN16;
		items->product.mppt.capabilities.historyVersion.properties.type				= VETI_UN8;
		items->product.mppt.capabilities.streetlightVersion.properties.type			= VETI_UN8;
#if !CFG_VEDIRECT_WITHOUT_MPPT_NETWORKING
		items->product.mppt.link.chargeVoltageSetpoint.properties.type				= VETI_UN16;
		items->product.mppt.link.networkMode.properties.type						= VETI_UN8;
		items->product.mppt.link.networkStatus.properties.type						= VETI_UN8;
		items->product.mppt.link.chargeCurrentPercentage.properties.type			= VETI_UN8;
		items->product.mppt.link.chargeCurrentLimit.properties.type					= VETI_UN16;
#endif
		break;
#endif
#ifdef CFG_VETEXT_WITH_INVERTER
	case classInverter:
		break;
#endif
	default:
		break;
	}
}

VeTinyItem * vetiItemFind(VetiItems * items, const VetiId id)
{
	const VetiId key = id;
	RegIdIndex * const regIdIndex = bsearch(&key, itemIndex, veDirectItemCount, sizeof(RegIdIndex), searchCompare);
	return regIdIndex ? (VeTinyItem *)(((char *) items) + regIdIndex->offset) : NULL;
}
