#ifndef ITEMS_HELPERS_H
#define ITEMS_HELPERS_H

#include <velib/types/ve_item.h>
#include <velib/types/ve_item_def.h>
#include <velib/types/variant_print.h>
#include <velib/ve_regs.h>

/**
 * To seperate gui logic / any other logic from the communication
 * the received values are stored in a tree. This struct
 * contains the field needed to create the tree.
 */
typedef struct
{
	VeRegId						veReg;
	size_t						item_offset;
#ifdef ITEM_TREE
	float						div;
	char const *				id;
	VeVariantUnitFmt const *	fmt;
	veBool						conditional;
#else
	un32						div;
#endif
} ItemInfo;

#ifdef ITEM_TREE
#define IOPT(div,...)		div, ##__VA_ARGS__

/* Dividers from VE_REG format to DBUS / SI format */
# define DIV_V		100			// 0.01V to V
# define DIV_I		10			// 0.1A to A
# define DIV_AH		10			// 0.1Ah to Ah
# define DIV_KWH	100			// 0.01kWh to kWh
# define DIV_WPV	100			// 0.01W to W
# define DIV_WB		1			// 1W to 1W
# define DIV_PCT	10			// 0.1% to %
# define DIV_SOC	100			// 0.01% to %
# define DIV_TTG	1.0f/60.0f	// minutes to seconds
# define DIV_T		100			// VREG is 0.01 K, this is converted to 0.01 degrees C

/**
 * Definitions how to make a textual representation of a value.
 * @see veVariantFmt in types/ve_variant_print.c.
 */
extern const VeVariantUnitFmt dcV;
extern const VeVariantUnitFmt dcI;
extern const VeVariantUnitFmt dcW;
extern const VeVariantUnitFmt dcT;
extern const VeVariantUnitFmt acV;
extern const VeVariantUnitFmt acI;
extern const VeVariantUnitFmt ampH;
extern const VeVariantUnitFmt kwh;
extern const VeVariantUnitFmt sec;
extern const VeVariantUnitFmt none;
extern const VeVariantUnitFmt percent;

#else
#define IOPT(div,...)		div

/* Dividers from VE_REG format to CAN format */
#define DIV_V 		1		// 0.01V
#define DIV_I 		1		// 0.1A
#define DIV_AH 		1		// 0.1Ah
#define DIV_KWH		1		// 0.01kWh
#define DIV_WPV		1		// 0.01W
#define DIV_WB		1		// 1W
#define DIV_SOC		100		// 0.01% to %
#endif


/* Conveniance function to set items to a static string.
 * It first sets the item to the buffer in value,
 * then copies the data to buf and then sets the item to buf.
 * This must be done like this for static strings, because when settings the item,
 * the current contents the item points to is compared with the new pointer buffer.
 * When the same pointer would be provided, the contents is always the same
 * and an item changed is not produced.
 * By first setting the value to a different buffer than the value of the item,
 * a check on the contents of the buffer is forced and an item changed produced
 * when the contents is different.
 */
void itemsSetBufferPtr(VeItem * item, VeVariant * value, void * buf, size_t len);

void itemsAddItemByInfo(VeItem *root, VeItem *item, ItemInfo const *itemInfo, VeItemSetterFun *fun, void * ctx);

void itemsSetItem(VeItem *item, VeVariant *v, ItemInfo const *itemInfo);

#endif // ITEMS_HELPERS_H

