#ifndef _ITEMS_BMV_H_
#define _ITEMS_BMV_H_

#include <velib/types/ve_item.h>

#include "vedirect_general_items.h"

typedef struct
{
	VeItem reason;						/* Raw BMV value */
	VeItem flags;						/* N2K format */

#ifdef ITEM_TREE
	VeItem relayState;					/* On/Off */
	VeItem alarm;						/* On/Off */
	VeItem lowVoltage;
	VeItem highVoltage;
	VeItem lowSoc;
	VeItem lowStarterVoltage;
	VeItem highStarterVoltage;
	VeItem lowTemperature;
	VeItem highTemperature;
	VeItem midVoltage;
#endif
} BmvAlarmItems;

typedef struct
{
	VeItem deepestDischarge;			/* [h1] DeepestDischarge [Ah] */
	VeItem lastDischarge;				/* [h2] Depth of last discharge [Ah], resetted at synchronisation. */
	VeItem averageDischarge;			/* [h3] AverageDischarge [Ah] */
	VeItem chargeCycles;				/* [h4] Number of dischargers, used for calculation of AverageDischarge.  */
	VeItem fullDischarges;				/* [h5] Number of full discharges, reaching 0.0% */
	VeItem totalAhDrawn;				/* [h6] Cumulative Ampere Hours */
	VeItem minimumVoltage;				/* [h7] Minimum Battery Volts measured */
	VeItem maximumVoltage;				/* [h8] Maximum Battery Volts measured */
	VeItem timeSinceLastFullCharge;		/* [h9] Days since last full charge */
	VeItem automaticSyncs;				/* [h10] number of synchronisatations */
	VeItem lowVoltageAlarms;			/* [h11] The number of low voltage alarms. */
	VeItem highVoltageAlarms;			/* [h12] The number of high voltage alarms */

	VeItem lowStarterVoltageAlarms;		/* [h13] The number of low starter battery voltage alarms. */
	VeItem highStarterVoltageAlarms;	/* [h14] The number of high starter battery voltage alarms */
	VeItem minimumStarterVoltage;		/* [h15] The minimum starter battery voltage. */
	VeItem maximumStarterVoltage;		/* [h16] The maximum starter battery voltage. */

	VeItem chargedEnergy;				/* [h18] The amount of charged energy. */
	VeItem dischargedEnergy;			/* [h19] The amount of discharged energy. */
} History;

typedef struct
{
	VeItem hasPower;
	VeItem hasTemperature;
	VeItem hasStarterVoltage;
	VeItem hasMidVoltage;
} Settings;

typedef struct VeBmvS
{
	DcStatus		dcStatusBattery;
	DcStatus		dcStatusAux;
	VeItem			consumedAmphours;
	VeItem			stateOfCharge;
	VeItem			timeToGo;
	VeItem			temperature;
	BmvAlarmItems	alarms;
	History			history;
	Settings		settings;
	un16			alarmFlags;
} VeBmvData;

#endif
