#ifndef VEDIRECT_SERVICE_H
#define VEDIRECT_SERVICE_H

#include <velib/types/ve_item.h>

#include <vedirect.h>
#include <veti.h>
#include <vedirect_veti.h>

#include "vedirect_solar_items.h"
#include "vedirect_bmv_items.h"
#include "vedirect_inverter_items.h"

typedef struct
{
	VeItem prodId;
	VeItem prodName;
	VeItem firmwareVersion;
	VeItem hardwareVersion;
	VeItem instance;
	VeItem connected;
	VeItem serial;

	union {
		VeBmvData bmv;
		VeSolar solar;
		VeInverter inverter;
	};
} VeItems;

struct VeDirectServiceS;
typedef struct VeDirectServiceS VeDirectService;

typedef struct
{
	void (*init)(VeDirectService *instance);
	void (*tick)(VeDirectService *instance);
	void (*invalidate)(VeDirectService *instance);
	void (*itemUpdate)(VeDirectService *instance, VetiId id, VeVariantValue value, VetiTypes type, veBool valid);
	veBool (*hasSerialNumber)(un16 pid);

	void (*connectedEvent)(VeDirectService *instance, un16 product);
	void (*disconnectedEvent)(VeDirectService *instance);

#ifdef ITEM_TREE
	const char *(*getServiceName)(void);
#endif
} VeDirectProduct;

struct VeDirectServiceS
{
	VeDirectInstance vd;
	VetiInstance veti;
	VdVetiInstance vdVeti;
	SubHandles vetiHandle;
	un16 vdTickCnt;

	void (*txFunc)(const un8 * data, un8 length, void * context);
	void *txContext;

	const VeDirectProduct *veDirectProduct;
	veBool connected;
	veBool registered;
	un16 prodId;
	un16 appVersion;
	char ser[12];

	VeItems items;

	VeItem root;

	VeItem *rootParent;

#ifdef ITEM_TREE
	char serviceName[256];
#endif
	un32 defaultInstanceId;
	const char *defaultServicePostfix;
};

#define item_offset(item) (offsetof(struct VeDirectServiceS, items.item))
#define instance_ptr(i,o) ((VeItem *)(((char *)i)+o))

/* rootParent is the item to which the vedirect service will attach its root item
 * when a device is detected. */
void veDirectServiceInit(VeDirectService *instance, VeItem *rootParent, un32 defaultInstanceId, const char *defaultServicePostfix);
void veDirectServiceTick(VeDirectService *instance);
static inline void veDirectServiceSetTx(VeDirectService *instance, void (*txFunc)(const un8 * data, un8 length, void * context), void *txContext) { instance->txFunc = txFunc; instance->txContext = txContext; }
static inline void veDirectServiceRxEvent(VeDirectService *instance, un8 byte) { vdRxData(&instance->vd, byte); }

#endif // VEDIRECT_SERVICE_H

