#ifndef ITEMS_SOLAR_H
#define ITEMS_SOLAR_H

#include <velib/types/ve_item.h>
#include <veti_mppt.h>

#include "vedirect_general_items.h"

typedef struct
{
	VeItem daysAvailable;
	VeItem lastError[4];
	VeItem yieldUser;
	VeItem yieldSystem;
	VeItem maxPvVoltage;
	VeItem maxBatteryVoltage;
	VeItem minBatteryVoltage;
	VeItem raw;
} SolarHistLast;

typedef struct
{
	VeItem yield;
	VeItem consumption;			/* For models with load output */
	VeItem maxBatteryVoltage;
	VeItem minBatteryVoltage;
	VeItem lastError[4];
	VeItem bulkTime;
	VeItem absTime;
	VeItem floatTime;
	VeItem maxPower;
	VeItem maxCurrent;
	VeItem maxPvVoltage;
	VeItem seqNr;
	VeItem raw;
} SolarHistDay;

typedef struct
{
	VeItem networkMode;
	VeItem networkStatus;
	VeItem chargeVoltageSet;
	VeItem chargeCurrentSet;
} Link;

typedef struct
{
	VeItem state;
	VeItem current;
} Load;

typedef struct
{
	VeItem chargeCurrentLimit;
	VeItem bmsPresent;
} SolarSettings;

typedef struct VeSolarS
{
	DcStatus		dcStatusBattery;
	DcStatus		dcStatusPvPanel;
	VeItem			error;
	VeItem			state;
	VeItem			relayState;
	Load			load;
	Link			link;
	SolarSettings	settings;
	Capabilities	capabilities;
	SolarHistLast	histLast;
	SolarHistDay	histDay[DAY_HISTORY_LENGTH];

	un16 day0SequenceNr;
	un8 historyDaysAvailable;

	MpptTotalsHistory historyTotal;
	MpptDayHistory historyDaily[DAY_HISTORY_LENGTH];

	/* For error suppressing */
	un8 errorCopy;
	un16 errorSuppressTimer;
} VeSolar;

#endif // ITEMS_SOLAR_H
