#include "items_helpers.h"

#include <string.h>
#include <velib/utils/ve_logger.h>

#define MODULE "items"

const VeVariantUnitFmt dcV		= {	2,	"V"		};
const VeVariantUnitFmt dcI		= {	1,	"A"		};
const VeVariantUnitFmt dcW		= {	0,	"W"		};
const VeVariantUnitFmt dcT		= {	0,	"C"		};
const VeVariantUnitFmt acV		= {	2,	"V"		};
const VeVariantUnitFmt acI		= {	1,	"A"		};
const VeVariantUnitFmt ampH		= {	1,	"Ah"	};
const VeVariantUnitFmt kwh		= {	2,	"kWh"	};
const VeVariantUnitFmt sec		= {	0,	"s"		};
const VeVariantUnitFmt none		= {	0,	""		};
const VeVariantUnitFmt percent	= {	1,	"%"		};

void itemsSetBufferPtr(VeItem * item, VeVariant * value, void * buf, size_t len)
{
	VeVariant v = *value;
	veItemOwnerSet(item, &v);
	memcpy(buf, v.value.CPtr, len);
	v.value.CPtr = buf;
	veItemOwnerSet(item, &v);
}

void itemsAddItemByInfo(VeItem *root, VeItem *item, ItemInfo const *itemInfo, VeItemSetterFun *fun, void *ctx)
{
#ifdef ITEM_TREE
	/* Not a dbus item */
	if (itemInfo->id == NULL)
		return;

	/* Check if exists */
	if (veItemByUid(root, itemInfo->id))
		return;

	/* Add */
	veItemAddChildByUid(root, itemInfo->id, item);
	veItemSetFmt(item, veVariantFmt, itemInfo->fmt);
//	veItemSetTimeout(item, itemInfo->timeout);

	if (itemInfo->div) {
		VeVariant v;
		v.type.tp = VE_FLOAT;
		veVariantInvalidate(&v);
		veItemLocalSet(item, &v);
	}

	if (fun) {
		veItemSetSetter(item, fun, ctx);
	}

	logI(MODULE, "added item %s", itemInfo->id);
#endif
}

void itemsSetItem(VeItem *item, VeVariant *v, ItemInfo const *itemInfo)
{
	if (veVariantIsValid(v)) {
		if (itemInfo->div) {
#ifdef ITEM_TREE
			veVariantToFloat(v);
			veVariantFloat(v, v->value.Float / itemInfo->div);
#else
			veVariantToN32(v);
			/* In C89, division of a negative number is undefined, so in this case, convert to positive first */
			if (v->type.tp == VE_UN32 || v->value.SN32 > 0)
				v->value.UN32 = (v->value.UN32 + itemInfo->div / 2) / itemInfo->div;
			else
				v->value.UN32 = -((sn32)(((un32)(-v->value.SN32) + itemInfo->div >> 1) / itemInfo->div));
#endif
			if (itemInfo->id)
				logI(MODULE, "Setting item %s to value %f", itemInfo->id, v->value.Float);
		}
		veItemOwnerSet(item, v);
	} else {
		veItemInvalidate(item);
	}
}

