#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

#include <velib/platform/console.h>
#include <velib/utils/ve_logger.h>
#include <velib/utils/ve_arg.h>

#define MODULE "console"

un32 timeOutArg;
extern long dbusInstance;

struct option consoleOptions[] =
{
	{"dbus-instance",required_argument,	0,								'i'},
	{"timeout",			required_argument,	0,							't'},
	{0,					0,					0,							0}
};

void consoleUsage(char* program)
{
	printf("%s\n", program);
	printf("\n");
	printf(" --dbus-instance\n");
	printf("   Instance to be used for the dbus service.\n");
	printf("   To discriminate multiple services of the same kind an instance number is\n");
	printf("   made available on the dbus.\n");
	printf("   Note: this number must be unique per service type, e.g. battery, not\n");
	printf("   only per program, since there might be multiple different batteries.\n");
	printf("\n");
	printf("  -t, --timeout\n");
	printf("   The time in seconds the program waits untill a device is connected. 0 is disabled.\n");
	printf("   If no device is found the program will stop if the timeout is expired.\n");
	printf("\n");
	pltOptionsUsage();
	printf("Victron Energy B.V.\n");
}

veBool consoleOption(int flag)
{
	char *p;

	switch(flag)
	{
	case 'i':
		dbusInstance = strtol(optarg, &p, 10);
		if (p == optarg || dbusInstance < 0 || dbusInstance > 65535) {
			logE(MODULE, "expected an instance in [0..65535]");
			pltExit(128);
		}
		break;


	case 't':
		errno = 0;
		timeOutArg = veArgInt(optarg);
		if (errno == EINVAL) {
			logE(MODULE, "wrong timeout argument");
			pltExit(128);
		}
	}

	return veTrue;
}

veBool consoleArgs(int argc, char *argv[])
{
	VE_UNUSED(argv);

	if (argc != 0)
	{
		printf("error - no arguments are expected - missing a '-' or '--'?");
		return veFalse;
	}

	return veTrue;
}
