#include "values.h"

#include <string.h>

#include <velib/base/ve_string.h>
#include <velib/types/ve_dbus_item.h>
#include <velib/utils/ve_logger.h>
#include <velib/platform/console.h>
#include <velib/types/variant_print.h>
#include <velib/types/ve_values.h>
#include <velib/platform/plt.h>
#include <velib/types/ve_item_def.h>

#include "serial_hal.h"

#define MODULE "values"

/* This file does not have to support multiple instances and can thus contain static variables */

/** The root of the tree to locate values and format them etc */
static VeItem root;

static VeItem processName;
static VeItem processVersion;
static VeItem connection;

extern un32 timeOutArg;
static un32 timeout;

/** dbus connection */
static struct VeDbus *dbus;

static char const *interface(void)
{
	VeVariant variant;

	serialDeviceInstance(&variant);
	veVariantToN32(&variant);

	if (variant.value.UN32 < 256)
		return "CAN";

	/* Serial ports */
	if (variant.value.UN32 < (256 + 32)) {
		return "VE.Direct";
	}

	return "USB";
}

void valuesUpdate(void)
{
	veDbusItemUpdate(dbus);
}

/** 50ms tick to invalidate items if there are not received in time */
void valuesTick(void)
{
	veItemTick(&root);

	if (timeout && --timeout == 0) {
		logE(MODULE, "timeout connecting device");
		pltExit(129);
	}
}

void serviceAboutToBeRemoved(VeItem * serviceRoot)
{
	VE_UNUSED(serviceRoot);
	logE(MODULE, "Stopping service.");
	pltExit(131);
}

static void serviceAdded(VeItem * serviceRoot)
{
	VeVariant variant;
	static const VeVariantUnitFmt none	= {	0,	""	};

	veItemSetAboutToRemoved(serviceRoot, serviceAboutToBeRemoved);

	/* App info */
	veItemAddChildByUid(serviceRoot, "Mgmt/ProcessName", &processName);
	veItemAddChildByUid(serviceRoot, "Mgmt/ProcessVersion", &processVersion);
	veItemAddChildByUid(serviceRoot, "Mgmt/Connection", &connection);
	veItemSetFmt(&processName, veVariantFmt, &none);
	veItemSetFmt(&processVersion, veVariantFmt, &none);
	veItemSetFmt(&connection, veVariantFmt, &none);
	veItemOwnerSet(&processName, veVariantStr(&variant, pltProgramName()));
	veItemOwnerSet(&processVersion, veVariantStr(&variant, pltProgramVersion()));
	veItemOwnerSet(&connection, veVariantStr(&variant, interface()));

	dbus = veDbusGetDefaultBus();
	if (!dbus) {
		logE(MODULE, "connected to dbus failed");
		pltExit(-4);
	}

	/* Device found */
	timeout = 0;

	veDbusItemInit(dbus, serviceRoot);
	veDbusChangeName(dbus, veItemId(serviceRoot));

	logI(MODULE, "connected to dbus (%s)", veItemId(serviceRoot));
}

void valuesInit(void)
{
	struct VeItem *root = veValueTree();

	timeout = timeOutArg * 20; /* 50 msec ticks */

	veItemSetChildAdded(root, serviceAdded);
}

