#include <string.h>

#include <velib/base/ve_string.h>
#include <velib/platform/plt.h>
#include <velib/platform/console.h>
#include <velib/types/ve_values.h>
#include <velib/types/ve_str.h>
#include <velib/utils/ve_logger.h>

#include "version.h"
#include "serial_hal.h"
#include "vedirect_service.h"
#include "values.h"

#define MODULE "task"

static VeDirectService service;
long dbusInstance = -1;

static void rxCallback(struct VeSerialPortS* port, un8 const *buf, un32 len)
{
	un32 idx;
	for (idx = 0; idx < len; ++idx)
		veDirectServiceRxEvent(&service, buf[idx]);
}

static void eventCallback(struct VeSerialPortS* port, VeSerialEvent event, char const *desc)
{
	VE_UNUSED(port);
	if (event == VE_SERIAL_EV_ERROR) {
		logE(MODULE, "Exiting because serial error occured: %s", desc);
		pltExit(150);
	}
}

static void txData (const un8 * const data, const un8 length, void * const context)
{
	VE_UNUSED(context);

	serialSendBuf(data, length);
}

void taskInit(void)
{
	VeVariant v;
	const char *uniqueServicePostfix;

	if (!serialConnect(rxCallback, eventCallback))
		pltExit(-1);

	/* allow passing the device instance as an argument */
	if (dbusInstance >= 0) {
		VeStr s;
		veVariantUn32(&v, dbusInstance);
		veStrNewFormat(&s, "instance%u", v.value.UN32);
		if (s.error)
			pltExit(50);
		uniqueServicePostfix = s.data;
	} else {
		if ((uniqueServicePostfix = strstr(pltGetSerialDevice(), "tty")) == NULL)
		{
			logE(MODULE, "uuuhh expected tty to be in the serial port");
			pltExit(-2);
		}
		/* allocate it, so it is on par with above */
		uniqueServicePostfix = strdup(uniqueServicePostfix);
		if (uniqueServicePostfix == NULL) {
			logE(MODULE, "out of memory....");
			pltExit(-3);
		}
		veVariantToN32(serialDeviceInstance(&v));
	}

	if (!veVariantIsValid(&v))
	{
		logE(MODULE, "failed to determine an unique number, see --dbus-instance");
		pltExit(60);
	}

	logI(MODULE, "using %s as unique service part", uniqueServicePostfix);
	veDirectServiceInit(&service, veValueTree(), v.value.UN32, uniqueServicePostfix);
	veDirectServiceSetTx(&service, txData, NULL);

	valuesInit();

	pltInterruptEnable();
}

void taskUpdate(void)
{
	valuesUpdate();
}

void taskTick(void)
{
	veDirectServiceTick(&service);
	valuesTick();
}

static char const version[] = VERSION_STR;
char const *pltProgramVersion(void)
{
	return version;
}
